#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

"""
Test module for IdaDataFrameObjects
"""
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from builtins import zip
from future import standard_library
standard_library.install_aliases()

import pandas
import pytest
import six

import ibmdbpy4nps
from ibmdbpy4nps import IdaDataBase


class Test_OpenDataFrameObject(object):

    def test_idadf_attr_idadb(self, idadf):
        assert isinstance(idadf._idadb, IdaDataBase)

    def test_idadf_attr_name(self, idadf, df):
        assert isinstance(idadf.name, six.string_types)
        assert idadf.name == idadf.schema + "." + "TEST_IBMDBPY"
        assert idadf.name == idadf.schema + "." + idadf.tablename

    def test_idadf_attr_schema(self, idadf):
        assert isinstance(idadf.schema, six.string_types)

    def test_idadf_attr_indexer(self, idadf):
        assert (isinstance(idadf.indexer, six.string_types)|(idadf.indexer is None))
        # TODO : Check more deeply the indexer

    def test_idadf_attr_loc(self, idadf):
        assert isinstance(idadf.loc, ibmdbpy4nps.indexing.Loc)

    def test_idadf_attr_internalstate(self, idadf):
        assert isinstance(idadf.internal_state, ibmdbpy4nps.internals.InternalState)

    def test_idadf_attr_type(self, idadf):
        assert isinstance(idadf.type, six.string_types)
        assert idadf.type == "Table"

    def test_idadf_atrr_dtypes(self, idadf, df):
        assert isinstance(idadf.dtypes, pandas.core.frame.DataFrame)
        assert len(idadf.dtypes) == len(idadf.columns)
        assert len(idadf.dtypes) == len(df.columns)

    def test_idadf_attr_index(self, idadf, df):
        # Ok, but what do we do if too big ?
        assert type(idadf.index) in [pandas.Int64Index, pandas.Index, pandas.RangeIndex]  # Not sure here
        assert list(idadf.index) == list(df.index)

    def test_idadf_attr_columns(self, idadf, df):
        assert isinstance(idadf.columns, pandas.core.index.Index)
        assert idadf.columns.equals(df.columns)

    def test_idadf_attr_axes(self, idadf):
        assert isinstance(idadf.axes, list)
        assert len(idadf.axes) == 2
        assert idadf.axes[1].equals(idadf.columns)
        assert list(idadf.axes[0]) == list(idadf.index)

    def test_idadf_attr_shape(self, idadf, df):
        assert isinstance(idadf.shape, tuple)
        assert len(idadf.shape) == 2
        assert idadf.shape[0] == len(idadf.index)
        assert idadf.shape[1] == len(idadf.columns)
        assert idadf.shape == df.shape

    def test_idadf_empty(self, idadb, df):
        idadb._create_table(df, "TEST_EMPTY_3496593727406047264076")
        to_test = ibmdbpy4nps.IdaDataFrame(idadb, "TEST_EMPTY_3496593727406047264076")
        assert(to_test.empty is True)
        idadb.drop_table("TEST_EMPTY_3496593727406047264076")

    def test_idadf_len(self, idadf, df):
        assert(len(idadf) == len(df))

    def test_idadf_iter(self, idadf, df):
        for idacol, col in zip(idadf, df):
            assert(idacol == col)


class Test_IdaDataFrameBehavior(object):
    def test_idadf_getitem_1_col_idadf(self, idadf):
        if len(idadf.columns) >= 1:
            newidadf = idadf[[idadf.columns[0]]]
            assert(isinstance(newidadf, ibmdbpy4nps.IdaDataFrame) is True)
            assert(len(newidadf.columns) == 1)
            assert(idadf.columns[0] == newidadf.columns[0])

            # We don't check of it is actually the corresponding column

            newidadf = idadf[[idadf.columns[-1]]]
            assert(isinstance(newidadf, ibmdbpy4nps.IdaDataFrame) is True)
            assert(len(newidadf.columns) == 1)
            assert(idadf.columns[-1] == newidadf.columns[0])

    def test_idadf_getitem_1_col_idadf_keyerror(self, idadf):
        with pytest.raises(KeyError):
            idadf[["NOTEXISTING_COLUMN_455849820205"]]

    def test_idadf_getitem_2_cols_idadf(self, idadf):
        if len(idadf.columns) >= 2:
            newidadf = idadf[[idadf.columns[0], idadf.columns[-1]]]
            assert(isinstance(newidadf, ibmdbpy4nps.IdaDataFrame) is True)
            assert(len(newidadf.columns) == 2)
            assert(idadf.columns[0] == newidadf.columns[0])
            assert(idadf.columns[-1] == newidadf.columns[-1])


    def test_idadf_getitem_2_cols_idadf_keyerror(self, idadf):
        with pytest.raises(KeyError):
            idadf[[idadf.columns[0], "NOTEXISTING_COLUMN_455849820205"]]

    # TODO : FIX If you select twice the same columns, only one with be taken into account
    # (This is because they are referenced in a dictionary, maybe force modifying the name of the columns)

    def test_idadf_getitem_all_cols_idadf(self, idadf):
        if len(idadf.columns) >= 1:
            newidadf = idadf[list(idadf.columns)]
            assert(isinstance(newidadf, ibmdbpy4nps.IdaDataFrame) is True)
            assert(len(newidadf.columns) == len(idadf.columns))
            assert(newidadf.shape == idadf.shape)

    def test_idadf_getitem_idaseries(self, idadf):
        if len(idadf.columns) >= 1:
            newidaseries = idadf[idadf.columns[0]]
            assert(isinstance(newidaseries, ibmdbpy4nps.IdaSeries))
            assert(len(newidaseries.columns) == 1)
            assert(idadf.columns[0] == newidaseries.columns[0])

            newidaseries = idadf[idadf.columns[-1]]
            assert(isinstance(newidaseries, ibmdbpy4nps.IdaDataFrame))
            assert(len(newidaseries.columns) == 1)
            assert(idadf.columns[-1] == newidaseries.columns[0])

    def test_idadf_getitem_idaseries_keyerror(self, idadf):
        with pytest.raises(KeyError):
            idadf["NOTEXISTING_COLUMN_455849820205"]

    def test_idadf_getitem_idaseries_keyerror_several_columns(self, idadf):
        if len(idadf.columns) >= 2:
            with pytest.raises(KeyError):
                idadf[idadf.columns[0], idadf.columns[1]]

    def test_idadf_getitem_slice(self, idadb, idadf, idadf_tmp):
        if len(idadf) > 10:
            newidadf = idadf[0:9]
            assert(len(newidadf) == 10)

        if len(idadf_tmp) > 10:
            idadb.add_column_id(idadf_tmp, destructive = True)
            newidadf_1 = idadf_tmp[0:9]
            newidadf_2 = idadf_tmp[0:9]
            assert(all(newidadf_1.head(10) == newidadf_2.head(10)))

    def test_idaseries_getitem_slice(self, idadb, idadf, idadf_tmp):
        # Set them as series first and do the same test as above
        if len(idadf.columns) >= 1:
            idadf = idadf[idadf.columns[0]]
            idadf_tmp = idadf_tmp[idadf_tmp.columns[0]]
            assert(isinstance(idadf, ibmdbpy4nps.IdaDataFrame))
            assert(isinstance(idadf_tmp, ibmdbpy4nps.IdaSeries))

            if len(idadf) > 10:
                newidadf = idadf[0:9]
                assert(len(newidadf) == 10)

    def test_idadf_setitem(self, idadf):
        pass

    def test_idadf_delitem(self, idadf):
        pass

    def test_idadf_filter_lt(self, idadf):
        pass

    def test_idadf_filter_le(self, idadf):
        pass

    def test_idadf_filter_eq(self, idadf):
        pass

    def test_idadf_filter_ne(self, idadf):
        pass

    def test_idadf_filter_ge(self, idadf):
        pass

    def test_idadf_filter_gt(self, idadf):
        pass

    def test_idadf_feature_add(self, idadf):
        pass

    def test_idadf_feature_radd(self, idadf):
        pass

    def test_idadf_feature_div(self, idadf):
        pass

    def test_idadf_feature_rdiv(self, idadf):
        pass

    def test_idadf_feature_floordiv(self, idadf):
        pass

    def test_idadf_feature_rfloordiv(self, idadf):
        pass

    def test_idadf_feature_mod(self, idadf):
        pass

    def test_idadf_feature_rmod(self, idadf):
        pass

    def test_idadf_feature_mul(self, idadf):
        pass

    def test_idadf_feature_rmul(self, idadf):
        pass

    def test_idadf_feature_neg(self, idadf):
        pass

    def test_idadf_feature_rpos(self, idadf):
        pass

    def test_idadf_feature_pow(self, idadf):
        pass

    def test_idadf_feature_rpow(self, idadf):
        pass

    def test_idadf_feature_sub(self, idadf):
        pass

    def test_idadf_feature_rsub(self, idadf):
        pass

class Test_DataBaseFeatures(object):

    def test_idadf_exists(self, idadf):
        assert(idadf.exists() is True)
        pass

    def test_idadf_is_view(self, idadf):
        assert(idadf.is_view() is False)
        pass

    def test_idadf_is_table(self, idadf):
        assert(idadf.exists() is True)
        pass

    def test_idadf_get_primary_key(self, idadf):
        pass

    def test_idadf_ida_query(self, idadf):
        pass

    def test_idadf_ida_scalar_query(self, idadf):
        pass


class Test_DataExploration(object):
    ### head
    # For head and tail we do not test if the rows match because
    # the order is not guaranteed anyway
    def test_idadf_head_default(self, idadb, idadf, df):
        sortkey = idadf.columns[0]
        if idadf._get_numerical_columns():
            sortkey = idadf._get_numerical_columns()[0]

        ida_head = idadf.head()
        assert isinstance(ida_head, pandas.core.frame.DataFrame)
        assert len(ida_head) == 5
        df_head = df.sort_values(sortkey).head()
        assert (ida_head[sortkey].tolist() == df_head[sortkey].tolist())

    def test_idadf_head_10(self, idadb, idadf, df):
        ida_head = idadf.head(10)
        assert isinstance(ida_head, pandas.core.frame.DataFrame)
        assert len(ida_head) == 10

    def test_idadf_head_10_sort(self, idadb, idadf, df):
        ida_head = idadf.head(10, sort=False)
        assert isinstance(ida_head, pandas.core.frame.DataFrame)
        assert len(ida_head) == 10

    def test_idadf_head_with_indexer(self, idadb, idadf_indexer, df):
        ida_head = idadf_indexer.head()
        sortby = len(df.columns)-1
        df_head = df.sort_values(df.columns[sortby]).head()
        assert isinstance(ida_head, pandas.core.frame.DataFrame)
        assert len(ida_head) == 5
        assert(ida_head[idadf_indexer.columns[sortby]].tolist() ==
                       df_head[df.columns[sortby]].tolist())

    def test_idadf_head_projected_3col(self, idadf, df):
        if len(idadf.columns) >= 4:
            columns = idadf.columns[1:4].tolist()
            newidadf = idadf[columns]

            sortkey = newidadf.columns[0]
            if newidadf._get_numerical_columns():
                sortkey = newidadf._get_numerical_columns()[0]

            ida_head = newidadf.head()

            df_sorted = df.sort_values(sortkey)
            df_head = df_sorted[columns].head()
            ida_sortkeys = ida_head[sortkey].tolist()
            df_sortkeys = df_head[sortkey].tolist()
            assert isinstance(ida_head, pandas.core.frame.DataFrame)
            assert len(ida_head) == 5
            # assert(ida_head[sortkey].tolist() == df_head[sortkey].tolist())
            assert([ round(x, 10) for x in  ida_sortkeys] == [ round(x, 10) for x in df_sortkeys ])

    def test_idadf_head_sorted(self, idadf, df):
        sortIdx = len(df.columns) - 1
        sortkey = idadf.columns[sortIdx]
        newidadf = idadf.sort(sortkey)
        ida_head = newidadf.head()

        df_head = df.sort_values(sortkey).head()

        assert(" ORDER BY " in newidadf.internal_state.get_state())
        assert isinstance(ida_head, pandas.core.frame.DataFrame)
        assert len(ida_head) == 5
        assert(ida_head[sortkey].tolist() == df_head[sortkey].tolist())

    def test_idadf_head_0(self, idadf):
        with pytest.raises(ValueError):
            idadf.head(0)

    def test_idadf_head_negative(self, idadf):
        with pytest.raises(ValueError):
            idadf.head(-1)

    ### tail
    def test_idadf_tail_default(self, idadb, idadf, df):
        sortkey = idadf.columns[0]
        if idadf._get_numerical_columns():
            sortkey = idadf._get_numerical_columns()[0]
        ida_tail = idadf.tail()
        assert isinstance(ida_tail, pandas.core.frame.DataFrame)
        assert len(ida_tail) == 5
        df_tail = df.sort_values(sortkey).tail()
        assert (ida_tail[sortkey].tolist() == df_tail[sortkey].tolist())

    def test_idadf_tail_10(self, idadb, idadf, df):
        ida_tail = idadf.tail(10)
        assert isinstance(ida_tail, pandas.core.frame.DataFrame)
        assert len(ida_tail) == 10

    def test_idadf_tail_10_sort(self, idadb, idadf, df):
        ida_tail = idadf.tail(10, sort=False)
        assert isinstance(ida_tail, pandas.core.frame.DataFrame)
        assert len(ida_tail) == 10

    def test_idadf_tail_with_indexer(self, idadb, idadf_indexer, df):
        ida_tail = idadf_indexer.tail()
        sortby = len(df.columns)-1
        df_head = df.sort_values(df.columns[sortby]).tail()
        assert isinstance(ida_tail, pandas.core.frame.DataFrame)
        assert len(ida_tail) == 5
        assert(ida_tail[idadf_indexer.columns[sortby]].tolist() ==
                       df_head[df.columns[sortby]].tolist())

    def test_idadf_tail_projected_3col(self, idadf, df):
        if len(idadf.columns) >= 4:
            columns = idadf.columns[1:4].tolist()
            newidadf = idadf[columns]

            sortkey = newidadf.columns[0]
            if newidadf._get_numerical_columns():
                sortkey = newidadf._get_numerical_columns()[0]

            ida_tail = newidadf.tail()

            df_sorted = df.sort_values(sortkey)
            df_tail = df_sorted[columns].tail()

            ida_sortkeys = ida_tail[sortkey].tolist()
            df_sortkeys = df_tail[sortkey].tolist()

            assert isinstance(ida_tail, pandas.core.frame.DataFrame)
            assert len(ida_tail) == 5
            # assert(ida_tail[sortkey].tolist() == df_tail[sortkey].tolist())
            assert([ round(x, 10) for x in  ida_sortkeys] == [ round(x, 10) for x in df_sortkeys ])

    @pytest.mark.skip(reason="tail on sorted dataframe fails in general, needs fixing first")
    def test_idadf_tail_sorted(self, idadf, df):
        sortIdx = len(df.columns) - 1
        sortkey = idadf.columns[sortIdx]
        newidadf = idadf.sort(sortkey)
        ida_tail = newidadf.tail()

        df_tail = df.sort_values(sortkey).tail()

        assert(" ORDER BY " in newidadf.internal_state.get_state())
        assert isinstance(ida_tail, pandas.core.frame.DataFrame)
        assert len(ida_tail) == 5
        assert(ida_tail[sortkey].tolist() == df_tail[sortkey].tolist())


    def test_idadf_tail_0(self, idadf):
        with pytest.raises(ValueError):
            idadf.tail(0)

    def test_idadf_tail_negative(self, idadf):
        with pytest.raises(ValueError):
            idadf.tail(-1)

    def test_idadf_pivot_table(self, idadf):
        pass

    def test_idadf_sort(self, idadf):
        pass

# no test
#__enter__
#__exit__



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

"""
In-Database Naive Bayes modelization and prediction.
Copies the interface of sklearn.naive_bayes
"""
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import
from builtins import dict
from builtins import str
from future import standard_library
standard_library.install_aliases()

from lazy import lazy
import ibmdbpy4nps
from ibmdbpy4nps.exceptions import IdaNaiveBayesError
import six

class NaiveBayes(object):
    """
    The Naive Bayes classification algorithm is a probabilistic classifier.
    It is based on probability models that incorporate strong independence
    assumptions. Often, the independence assumptions do not have an impact
    on reality. Therefore, they are considered naive.

    The NaiveBayes class provides an interface for using the NAIVEBAYES
    and PREDICT_NAIVEBAYES IDAX/INZA methods.
    """

    def __init__(self, modelname = None, disc = None, bins = None):
        """
        Constructor for NaiveBayes model objects

        Parameters
        ----------
        modelname : str, optional
            The name of the Naive Bayes model that will be built.
            Should contain only alphanumerical characters and underscores.
            All lower case characters will be converted to upper case characters.
            If no name is specified, it will be generated automatically.
            If the parameter corresponds to an existing model in the database,
            it is replaced during the fitting step.

        disc : str, optional, default: ew
            Determine the automatic discretization of all continuous attributes. 
            The following values are allowed: ef, em, ew, and ewn.

                * disc=ef
                    Equal-frequency discretization.
                    An unsupervised discretization algorithm that uses the equal
                    frequency criterion for interval bound setting.
                * disc=em
                    Minimal entropy discretization.
                    An unsupervised discretization algorithm that uses the minimal
                    entropy criterion for interval bound setting.
                * disc=ew (default)
                    Equal-width discretization.
                    An unsupervised discretization algorithm that uses the equal
                    width criterion for interval bound setting.
                * disc=ewn
                    Equal-width discretization with nice bucket limits.
                    An unsupervised discretization algorithm that uses the equal
                    width criterion for interval bound setting.

        bins : int, optional, default : 10
            Number of bins for numeric columns.
            
        Attributes
        ----------
        
        _idadf: Get set at fit step, IdaDataFrame used as input to fit the model;
        
        _idadb: Get set at fit step, IdaDataBase object holding the connection to Db2;
        
        _column_id: Get set at fit step, name of the column of the input table which identifies the transaction or class ID.;
        
        target: Get set at fit step; str
            The column of the input table that represents the class
        
        incolumn: Get set at fit step; str, optional
            The columns of the input table that have specific properties,
            which are separated by a semi-colon (;). Each column is succeeded
            by one or more of the following properties:

                * By type nominal (':nom') or by type continuous (':cont'). By default,
                  numerical types are continuous, and all other types are nominal.
                * By role ':id', ':target', ':input', or ':ignore'.

        coldeftype: Get set at fit step; str, optional
            The default type of the input table columns.
            The following values are allowed: 'nom' and 'cont'.
            If the parameter is not specified, numeric columns are continuous,
            and all other columns are nominal.
        
        coldefrole: Get set at fit step; str, optional
            The default role of the input table columns.
            The following values are allowed: 'input' and 'ignore'.
            If the parameter is not specified, all columns are input columns.
        
        colpropertiestable: Get set at fit step; str, optional
            The name of the input table where the properties of the columns of the input table
            are stored. If this parameter is not specified, the column properties of the input table
            column properties are detected automatically.
        
        outable: Get set at predict step; str, optional
            The name of the output table where the predictions are stored. If
            this parameter is not specified, it is generated automatically. If
            the parameter corresponds to an existing table in the database, it
            will be replaced.
        
        outtableProb: Get set at predict step; str, optional
            The output table where the probabilities for each of the classes are stored.
            If this parameter is not specified, the table is not created. If
            the parameter corresponds to an existing table in the database, it
            will be replaced.
        
        mestimation: Get set at predict step; flag, default: False
            A flag that indicates the use of m-estimation for probabilities.
            This kind of estimation might be slower than other ones, but it
            might produce better results for small or unbalanced data sets.
        
        modelname: see parameters;
        
        disc: see parameters;
        
        bins: see parameters;
        

        Returns
        -------
            The NaiveBayes object, ready to be used for fitting and prediction.

        Examples
        --------
        >>> idadb = IdaDataBase("BLUDB-TEST")
        >>> idadf = IdaDataFrame(idadb, "IRIS")
        >>> bayes = NaiveBayes("NAIVEBAYES_TEST")
        >>> bayes.fit(idadf, column_id="ID", target="species")
        >>> bayes.predict(idadf, outtable="IRIS_PREDICTION", outtableProb="IRIS_PREDICTIONPROB")

        Notes
        -----
            Inner parameters of the model can be printed and modified by using 
            get_params and set_params. But we recommend creating a new 
            NaiveBayes model instead of modifying an existing model.
        """
        # Get set at fit step
        self._idadf = None
        self._idadb = None
        self._column_id = None
        self.target = None
        self.incolumn = None
        self.coldeftype = None
        self.coldefrole = None
        self.colpropertiestable = None

        # Get set at predict step
        self.outtable = None
        self.outtableProb = None
        self.mestimation = None

        self.modelname = ibmdbpy4nps.utils.check_modelname(modelname)
        self.disc = disc
        self.bins = bins


    @lazy
    def labels_(self):
        """
        Return the labels of the classification if available
        """
        try:
            return self.predict(self._idadf, self._column_id)
        except:
            raise AttributeError(str(self.__class__) + " object has no attribute 'labels_'")

    def get_params(self):
        """
        Return the parameters of the Naive Byes model.
        """
        print(dir(self))
        params = dict()
        params['modelname'] = self.modelname
        params['disc'] = self.disc
        params['bins'] = self.bins

        params['target'] = self.target
        params['incolumn'] = self.incolumn
        params['coldeftype'] = self.coldeftype
        params['coldefrole'] = self.coldefrole
        params['colpropertiestable'] = self.colpropertiestable

        params['outtable'] = self.outtable
        params['outtableProb'] = self.outtableProb
        params['mestimation'] = self.mestimation

        return params

    def set_params(self, **params):
        """
        Modify the parameters of the Naive Bayes model.
        """
        if not params:
            # Simple optimisation to gain speed (inspect is slow)
            return self
        valid_params = self.get_params()
        for key, value in six.iteritems(params):
            if key not in valid_params:
                raise ValueError('Invalid parameter %s for estimator %s' %
                                     (key, self.__class__.__name__))
            setattr(self, key, value)
        return self

    def fit(self, idadf, target, column_id="ID", incolumn=None, coldeftype=None,
            coldefrole=None, colpropertiestable=None, verbose=False):
        """
        Create a Naive Bayes model from an IdaDataFrame.

        Parameters
        ----------
        idadf : IdaDataFrame
            The IdaDataFrame to be used as input.

        target : str
            The column of the input table that represents the class

        column_id : str, default: "ID
            The column of the input table that identifies the transaction ID.

        incolumn : str, optional
            The columns of the input table that have specific properties,
            which are separated by a semi-colon (;). Each column is succeeded
            by one or more of the following properties:

                * By type nominal (':nom') or by type continuous (':cont'). By default,
                  numerical types are continuous, and all other types are nominal.
                * By role ':id', ':target', ':input', or ':ignore'.

            If this parameter is not specified, all columns of the input table have default properties.

        coldeftype : str, optional
            The default type of the input table columns.
            The following values are allowed: 'nom' and 'cont'.
            If the parameter is not specified, numeric columns are continuous,
            and all other columns are nominal.

        coldefrole : str, optional
            The default role of the input table columns.
            The following values are allowed: 'input' and 'ignore'.
            If the parameter is not specified, all columns are input columns.

        colpropertiestable : str, optional
            The input IdaDataFrame or the name of the table where the properties of the
            columns of the input IdaDataFrame (idadf) are stored.
            The table name should contain only alphanumerical characters and underscores.
            All lower case characters will be converted to upper case characters.
            If this parameter is not specified, the column properties of the input table
            column properties are detected automatically.

        verbose : bool, default: False
            Verbosity mode.
        """

        # Some basic checks
        if not isinstance(idadf, ibmdbpy4nps.IdaDataFrame):
            raise TypeError("Argument should be an IdaDataFrame")
        if target not in idadf.columns:
            raise ValueError("Target is not a column in " + idadf.name)

        idadf._idadb._check_procedure("NAIVEBAYES", "Naive Bayes")

        # Check the ID
        if column_id not in idadf.columns:
            raise ValueError("No id columns is available in IdaDataFrame:" + column_id +
                             ". Either create a new ID column using add_column_id function" +
                             " or give the name of a column that can be used as ID")

        self._idadb = idadf._idadb
        self._idadf = idadf
        self._column_id = column_id

        self.target = target
        self.incolumn = incolumn
        self.coldeftype = coldeftype
        self.coldefrole = coldefrole
        if not (colpropertiestable is None or isinstance(colpropertiestable, ibmdbpy4nps.frame.IdaDataFrame)):
            colpropertiestable = ibmdbpy4nps.utils.check_tablename(colpropertiestable)
        self.colpropertiestable = colpropertiestable

        # Check or create a model name, drop it if it already exists.
        if self.modelname is None:
            self.modelname = idadf._idadb._get_valid_modelname('NAIVEBAYES_')
        else:
            self.modelname = ibmdbpy4nps.utils.check_modelname(self.modelname)
            if idadf._idadb.exists_model(self.modelname):
                idadf._idadb.drop_model(self.modelname)

        # Create a temporay view
        # TODO: Why do we need actually to create a view ?
        tmp_view_name = idadf.internal_state._create_view()
        # tmp_view_name = idadf.internal_state.current_state
        
        #if "." in tmp_view_name:
            #tmp_view_name = tmp_view_name.split('.')[-1]

        try:
            idadf._idadb._call_stored_procedure("NAIVEBAYES ",
                                                 model = self.modelname,
                                                 intable = tmp_view_name,
                                                 id = self._column_id,
                                                 target = self.target,
                                                 incolumn = self.incolumn,
                                                 coldeftype = self.coldeftype,
                                                 coldefrole = self.coldefrole,
                                                 colPropertiesTable = self.colpropertiestable,
                                                 disc = self.disc,
                                                 bins = self.bins)
        except:
            raise
        finally:
            idadf.internal_state._delete_view()
            idadf.commit()

        self._retrieve_NaiveBayes_Model(self.modelname, verbose)

        if verbose is True:
            self.describe()

        return

    def predict(self, idadf, column_id=None, outtable=None, outtableProb=None,
                mestimation=False):
        """
        Use the Naive Bayes predict stored procedure to apply a Naive Bayes model
        to generate classification predictions for a data set.

        Parameters
        ----------
        idadf : IdaDataFrame
             IdaDataFrame to be used as input.

        column_id : str, optional
            The column of the input table that identifies a unique instance ID.
            By default, the same id column that is specified in the stored
            procedure to build the model.

        outtable : str, optional
            The name of the output table where the predictions are stored.
            It should contain only alphanumerical characters and underscores.
            All lower case characters will be converted to upper case characters.
            If this parameter is not specified, it is generated automatically. If
            the parameter corresponds to an existing table in the database, it
            will be replaced.

        outtableProb : str, optional
            The name of the output table where the probabilities for each of the classes are stored.
            It should contain only alphanumerical characters and underscores.
            All lower case characters will be converted to upper case characters.
            If this parameter is not specified, the table is not created.
            If the parameter corresponds to an existing table in the database, it
            will be replaced.

        mestimation : flag, default: False
            A flag that indicates the use of m-estimation for probabilities.
            This kind of estimation might be slower than other ones, but it
            might produce better results for small or unbalanced data sets.

        Returns
        -------
        IdaDataFrame
            IdaDataFrame containing the classification decision for each
            datapoints referenced by their ID.
        """
        if not isinstance(idadf, ibmdbpy4nps.IdaDataFrame):
            raise TypeError("Argument should be an IdaDataFrame")

        idadf._idadb._check_procedure("PREDICT_NAIVEBAYES", "Prediction for Naive Bayes")

        # Check the ID
        if column_id is None :
            column_id = self._column_id
        if column_id not in idadf.columns:
            raise ValueError("No id columns is available in IdaDataFrame:" + column_id +
                             ". Either create a new ID column using add_column_id function" +
                             " or give the name of a column that can be used as ID")

        if self._idadb is None:
            raise IdaNaiveBayesError("The Naive Bayes model was not trained before.")

        # Check or create an outtable name, drop it if it already exists.
        if outtable is None:
            outtable = idadf._idadb._get_valid_tablename('PREDICT_NAIVEBAYES_')
        else:
            outtable = ibmdbpy4nps.utils.check_tablename(outtable)
            if idadf._idadb.exists_table(outtable):
                idadf._idadb.drop_table(outtable)

        if outtableProb is not None:
            outtableProb = ibmdbpy4nps.utils.check_tablename(outtableProb)
            if idadf._idadb.exists_table(outtableProb):
                idadf._idadb.drop_table(outtableProb)

        self.outtable = outtable
        self.outtableProb = outtableProb
        self.mestimation = mestimation

        # Create a temporay view
        tmp_view_name = idadf.internal_state._create_view()
        # tmp_view_name = idadf.internal_state.current_state
        
        #if "." in tmp_view_name:
            #tmp_view_name = tmp_view_name.split('.')[-1]

        try:
            idadf._idadb._call_stored_procedure("PREDICT_NAIVEBAYES ",
                                                 model = self.modelname,
                                                 intable = tmp_view_name,
                                                 id = column_id,
                                                 outtable = self.outtable,
                                                 outtableProb = self.outtableProb,
                                                 mestimation = self.mestimation
                                                 )
        except:
            raise
        finally:
            idadf.internal_state._delete_view()
            idadf._idadb._autocommit()

        self.labels_ = ibmdbpy4nps.IdaDataFrame(idadf._idadb, self.outtable)
        return self.labels_

    def fit_predict(self, idadf, column_id="ID", incolumn=None, coldeftype=None,
                    coldefrole=None, colprepertiesTable=None, outtable=None,
                    outtableProb=None, mestimation=False, verbose=False):
        """
        Convenience function for fitting the model and using it to make 
        predictions about the same dataset. See to fit and predict 
        documentation for an explanation about their attributes.
        """
        self.fit(idadf, column_id, incolumn, coldeftype, coldefrole, colprepertiesTable, verbose)
        return self.predict(idadf, column_id, outtable, outtableProb, mestimation)

    def describe(self, detail=True):
        """
        Parameters
        ----------
        detail: bool, optional. True by default.
        
        Returns
        -------
        Return a description of Naives Bayes.
        If False, only the a priori probabilities of each class are returned, 
        displayed in table format. If True, then intermediary tables are shown.
        """
        if self._idadb is None:
            return self.get_params
        else:
            try:
                if detail:
                    self._retrieve_NaiveBayes_Model(self.modelname, verbose=True)
                # PRINT_MODEL not supported for Naive Bayes on Netezza‚
                if not self._idadb._is_netezza_system():
                     # res = self._idadb.ida_query("CALL IDAX.PRINT_MODEL('model = " + self.modelname +"')")
                    self._idadb._call_stored_procedure("PRINT_MODEL ", model = self.modelname)
            except:
                raise
            return


    def _retrieve_NaiveBayes_Model(self, modelname, verbose=False):
        """
        Retrieve information about the model to print the results. The Naive 
        Bayes IDAX/INZA function stores its result in 2 tables:

            * <MODELNAME>_MODEL
            * <MODELNAME>_DISCRANGES

        Parameters
        ----------
        modelname : str
            The name of the model that is retrieved.

        verbose : bol, default: False
            Verbosity mode.

        Notes
        -----
        Needs better formatting instead of printing the tables.
        """
        modelname = ibmdbpy4nps.utils.check_modelname(modelname)

        if self._idadb is None:
            raise IdaNaiveBayesError("The Naive Bayes model was not trained before.")

        if self._idadb._is_netezza_system():
            modeltable_prefix = "INZA.NZA_META_"
            disctable_postfix =  "_DISC"
        else:
            modeltable_prefix = ""
            disctable_postfix =  "_DISCRANGES"

        model_main = self._idadb.ida_query('SELECT * FROM ' + modeltable_prefix + modelname + '_MODEL')
        model_main.columns = ['ATTRIBUTE', 'VAL', 'CLASS', 'CLASSVALCOUNT', 'ATTRCLASSCOUNT',
       'CLASSCOUNT', 'TOTALCOUNT']
        model_main.columns = [x.upper() for x in model_main.columns]

        disc = self._idadb.ida_query('SELECT * FROM ' + modeltable_prefix + modelname + disctable_postfix)
        disc.columns = ['COLNAME', 'BREAK']
        disc.columns = [x.upper() for x in disc.columns]

        if verbose is True:
            print("MODEL")
            print(model_main)
            print("DISCRANGES")
            print(disc)

        return


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

"""
Test module for private methods of IdaDataFrameObjects
"""
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from builtins import zip
from builtins import str
from future import standard_library
standard_library.install_aliases()

import pytest
import six

from ibmdbpy4nps import IdaDataFrame
from ibmdbpy4nps.exceptions import IdaDataBaseError


class Test_IdaDataBase_PrivateMethods(object):

    def test_idadb_upper_columns(self, idadb, df):
        df_upper = idadb._upper_columns(df)
        for column in df_upper.columns:
            assert(column == column.upper())

    def test_idadb_get_name_and_schema(self, idadb):
        tup = idadb._get_name_and_schema("SCHEMA.TABLE")
        assert(tup[0] == "SCHEMA")
        assert(tup[1] == "TABLE")

    def test_idadb_get_name_and_schema_no_schema(self, idadb):
        tup = idadb._get_name_and_schema("TABLE")
        assert(tup[0] == idadb.current_schema)
        assert(tup[1] == "TABLE")

    def test_idadb_get_valid_tablename_default(self, idadb):
        tablename = idadb._get_valid_tablename()
        assert(isinstance(tablename, six.string_types))
        assert("DATA_FRAME_" in tablename)

    def test_idadb_get_valid_tablename_custom(self, idadb):
        tablename = idadb._get_valid_tablename("MY_PERSONAL_PREFIX")
        assert("MY_PERSONAL_PREFIX" in tablename)

    def test_idadb_get_valid_tablename_error(self, idadb):
        with pytest.raises(ValueError):
            idadb._get_valid_tablename("INCORRECT_PREFIX_ß%$")

    def test_idadb_get_valid_viewname_default(self, idadb):
        viewname = idadb._get_valid_viewname()
        assert(isinstance(viewname, six.string_types))
        assert("VIEW_" in viewname)

    def test_idadb_get_valid_viewname_custom(self, idadb):
        viewname = idadb._get_valid_viewname("MY_PERSONAL_PREFIX")
        assert("MY_PERSONAL_PREFIX" in viewname)

    def test_idadb_get_valid_viewname_error(self, idadb):
        with pytest.raises(ValueError):
            idadb._get_valid_viewname("INCORRECT_PREFIX_ß%$")

    def test_idadb_get_valid_modelname_default(self, idadb):
        modelname = idadb._get_valid_modelname()
        assert(isinstance(modelname, six.string_types))
        assert("MODEL_" in modelname)

    def test_idadb_get_valid_modelname_custom(self, idadb):
        modelname = idadb._get_valid_modelname("MY_PERSONAL_PREFIX")
        assert("MY_PERSONAL_PREFIX" in modelname)

    def test_idadb_get_valid_modelname_error(self, idadb):
        with pytest.raises(ValueError):
            idadb._get_valid_modelname("INCORRECT_PREFIX_ß%$")

    def test_idadb_create_table(self, idadb, df):
        try : idadb.drop_table("CREATE_TABLE_TEST_585960708904")
        except : pass
        idadb._create_table(df, "CREATE_TABLE_TEST_585960708904")
        ida = IdaDataFrame(idadb, "CREATE_TABLE_TEST_585960708904")
        assert(all([str(x) == str(y) for x,y in zip(df.columns, ida.columns)]))
        assert(idadb.exists_table("CREATE_TABLE_TEST_585960708904") == 1)
        idadb.drop_table("CREATE_TABLE_TEST_585960708904")

    def test_idadb_create_view(self, idadb, df):
        try : idadb.drop_table("CREATE_VIEW_TEST_585960708904")
        except: pass
        try : idadb.drop_view("VIEW_TEST_585960708904")
        except: pass
        idadb._create_table(df, "CREATE_VIEW_TEST_585960708904")
        ida = IdaDataFrame(idadb, "CREATE_VIEW_TEST_585960708904")
        idadb._create_view(ida, "VIEW_TEST_585960708904")
        idadb.drop_table("CREATE_VIEW_TEST_585960708904")
        idadb.drop_view("VIEW_TEST_585960708904")

    # Make test when it fails too
    def test_idadb_insert_into_database(self, idadb, df):
        try : idadb.drop_table("INSERT_TEST_585960708904")
        except: pass
        idadb._create_table(df, "INSERT_TEST_585960708904")
        ida = IdaDataFrame(idadb, "INSERT_TEST_585960708904")
        assert(ida.shape[0] == 0)
        idadb._insert_into_database(df.loc[[1]], idadb.current_schema, "INSERT_TEST_585960708904")
        del ida.shape
        assert(ida.shape[0] == 1)
        idadb.drop_table("INSERT_TEST_585960708904")

    def test_idadb_check_procedure(self, idadb):
        # This test work of course only if KMEANS is available
        bol = idadb._check_procedure('KMEANS')
        assert(bol is True)

    def test_idadb_check_procedure_fail(self, idadb):
        with pytest.raises(IdaDataBaseError):
            idadb._check_procedure('NOTEXISTINGPROCEDURE_495906823812')

    def test_idadb_reset_attributes(self, idadb):
        idadb.useless_attribute = "test"
        assert(hasattr(idadb, "useless_attribute"))
        idadb._reset_attributes("useless_attribute")
        assert(not hasattr(idadb, "useless_attribute"))
        idadb.useless_attribute = "test"
        assert(hasattr(idadb, "useless_attribute"))
        idadb._reset_attributes(["useless_attribute"])
        assert(not hasattr(idadb, "useless_attribute"))

    def test_idadb_reset_attributes_not_existing(self, idadb):
        assert(not hasattr(idadb, "NOTEXISTINGATTRIBUTE_374965038285"))
        idadb._reset_attributes("NOTEXISTINGATTRIBUTE_374965038285")
        assert(not hasattr(idadb, "NOTEXISTINGATTRIBUTE_374965038285"))

    def test_idadb_reset_attributes_multiple(self, idadb):
        idadb.useless_attribute1 = "test"
        idadb.useless_attribute2 = "test"
        assert(hasattr(idadb, "useless_attribute1"))
        assert(hasattr(idadb, "useless_attribute2"))
        assert(not hasattr(idadb, "useless_attribute3"))
        idadb._reset_attributes(["useless_attribute1", "useless_attribute2", "useless_attribute3"])
        # Note : One is actually not existing
        assert(not hasattr(idadb, "useless_attribute1"))
        assert(not hasattr(idadb, "useless_attribute2"))
        assert(not hasattr(idadb, "useless_attribute3"))

    def test_idadb_reset_retrieve_cache(self, idadb):
        idadb.my_custom_cache = "test"
        cache = idadb._retrieve_cache("my_custom_cache")
        assert(cache == idadb.my_custom_cache)

    def test_idadb_reset_retrieve_cache_not_existing(self, idadb):
        cache = idadb._retrieve_cache("NOTEXISTINGCACHE_43845696794939")
        assert(cache is None)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

"""
idaSeries
"""

# Ensure Python 2 compatibility
from __future__ import print_function
from __future__ import division
from __future__ import unicode_literals
from __future__ import absolute_import
from builtins import super
from future import standard_library
standard_library.install_aliases()

from copy import deepcopy

from lazy import lazy

import ibmdbpy4nps

class IdaSeries(ibmdbpy4nps.IdaDataFrame):
    """
    IdaSeries can be considered as a different version of IdaDataFrame
    objects that have only one column and can be thus represented
    as pandas.Series to the user.
    """
    def __init__(self, idadb, tablename, indexer, column):
        super(IdaSeries, self).__init__(idadb, tablename, indexer)
        self.column = column

    ##### legacy
    @lazy
    def columns(self):
        return [self.column]

# TODO : Override all methods for which the behavior, i.e. the output is
# different in comparision with the one of an IdaDataFrame. For now the
# disjunction are implemented on the functions only.

    def min(self):
        result = super(IdaSeries, self).min()
        #import pdb; pdb.set_trace()
        return result[0]
        
    def max(self):
        result = super(IdaSeries, self).max()
        #import pdb; pdb.set_trace()
        return result[0]

    def _clone(self):
        """
        Clone an IdaSeries.
        """
        newida = IdaSeries(self._idadb, self._name, self.indexer, self.column)
        newida.internal_state.name = deepcopy(self.internal_state.name)
        newida.internal_state.ascending = deepcopy(self.internal_state.ascending)
        #newida.internal_state.views = deepcopy(self.internal_state.views)
        newida.internal_state._views = deepcopy(self.internal_state._views)
        newida.internal_state._cumulative = deepcopy(self.internal_state._cumulative)
        newida.internal_state.order = deepcopy(self.internal_state.order)
        return newida


# -*- coding: utf-8 -*-
"""
Created on Mon Nov 23 09:48:18 2015

@author: efouche
"""
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from builtins import dict
from future import standard_library
standard_library.install_aliases()

from collections import OrderedDict

from ibmdbpy4nps.feature_selection.entropy import entropy

from ibmdbpy4nps.internals import idadf_state
from ibmdbpy4nps.utils import timed

import numpy as np 
import pandas as pd

from ibmdbpy4nps.feature_selection.private import _check_input


@idadf_state
@timed
def gain_ratio(idadf, target = None, features = None, symmetry=True, ignore_indexer=True):
    """
    Compute the gain ratio coefficients between a set of features and a 
    set of target in an IdaDataFrame. 
    
    Parameters
    ----------
    idadf : IdaDataFrame
    
    target : str or list of str, optional
        A column or list of columns against to be used as target. Per default, 
        consider all columns
    
    features : str or list of str, optional
        A column or list of columns to be used as features. Per default, 
        consider all columns. 
        
    symmetry : bool, default: True
        If True, compute the symmetric gain ratio as defined by
        [Lopez de Mantaras 1991]. Otherwise, the asymmetric gain ratio. 
    
    ignore_indexer : bool, default: True
        Per default, ignore the column declared as indexer in idadf
        
    Returns
    -------
    Pandas.DataFrame or Pandas.Series if only one target
    
    Notes
    -----
    Input columns as target and features should be categorical, otherwise 
    this measure does not make much sense. 
    
    Examples
    --------
    >>> idadf = IdaDataFrame(idadb, "IRIS")
    >>> gain_ratio(idadf)
    """
    # Check input 
    target, features = _check_input(idadf, target, features, ignore_indexer)
    
    entropy_dict = dict()
    length = len(idadf)
    values = OrderedDict()
    corrector = length*np.log(length)
        
    for t in target:
        if t not in values:
            values[t] = OrderedDict()
        features_notarget = [x for x in features if (x != t)]
        
        for feature in features_notarget:
            if feature not in values:
                values[feature] = OrderedDict()      
                
            if t not in values[feature]:    # i.e. it was not already computed 
                if t not in entropy_dict:
                    entropy_dict[t] = entropy(idadf, t, mode = "raw")
                if feature not in entropy_dict:
                    entropy_dict[feature] = entropy(idadf, feature, mode = "raw")
                    
                join_entropy = entropy(idadf,  [t] + [feature], mode = "raw")     
                disjoin_entropy = entropy_dict[t] + entropy_dict[feature]
                info_gain = (disjoin_entropy - join_entropy)
            
                if symmetry:
                    gain_ratio = (info_gain + corrector)/(disjoin_entropy + 2*corrector) # 2* because symmetric
                    values[t][feature] = gain_ratio
                    if feature in target:
                        values[feature][t] = gain_ratio
                else:
                    gain_ratio_1 = (info_gain + corrector)/(entropy_dict[t] + corrector)
                    values[t][feature] = gain_ratio_1
                    if feature in target:
                        gain_ratio_2 = (info_gain + corrector)/(entropy_dict[feature] + corrector)
                        values[feature][t] = gain_ratio_2
             
    ### Fill the matrix
    result = pd.DataFrame(values).fillna(np.nan)
    result = result.dropna(axis=1, how="all")
        
    if len(result.columns) > 1:
        order = [x for x in result.columns if x in features] + [x for x in features if x not in result.columns]
        result = result.reindex(order)
       
    if len(result.columns) == 1:
        if len(result) == 1:
            result = result.iloc[0,0]
        else:
            result = result[result.columns[0]].copy()
            result.sort_values(ascending = True)
    else:
        result = result.fillna(1)
            
    return result


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

""" Utility functions """

# Python 2 compatibility
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import
from builtins import range
from builtins import input
from future import standard_library
standard_library.install_aliases()

import sys
import os
import warnings
from time import time
from functools import wraps

import six
import pandas as pd
from copy import deepcopy

#-----------------------------------------------------------------------------
# Environment variable setter

def set_verbose(is_verbose):
    """
    Set the environment variable “VERBOSE” to ‘TRUE’ or ‘FALSE’. If it is set 
    to ‘TRUE’, all SQL request are printed in the console.
    """
    if is_verbose is True:
        os.environ['VERBOSE'] = 'True'
    elif is_verbose is False:
        os.environ['VERBOSE'] = 'False'
    else:
        raise ValueError("is_verbose should be a boolean")

def set_autocommit(is_autocommit):
    """
    Set the environment variable “AUTCOMMIT” to ‘TRUE’ or ‘FALSE’. If it is set 
    to ‘TRUE’, all operations are committed automatically.
    """
    if is_autocommit is True:
        os.environ['AUTOCOMMIT'] = 'True'
    elif is_autocommit is False:
        os.environ['AUTOCOMMIT'] = 'False'
    else:
        raise ValueError("is_autocommit should be a boolean")

#-----------------------------------------------------------------------
# Performance measurement wrapper

def timed(function):
    """
    Measure the elapsed time of custom functions of the package. Should be used 
    as a decorator.
    """
    @wraps(function)
    def wrapper(*args, **kwds):
        """Calculate elapsed time in seconds"""
        start = time()
        result = function(*args, **kwds)
        elapsed = time() - start
        if os.environ['VERBOSE'] == 'True':
            print("Execution time: %s seconds." %elapsed)
        return result
    return wrapper

def query_yes_no(question, default=None):
    """
    Ask a yes/no question via raw_input() and return its answer.

    Parameters
    ----------
    question : str
        Question to be asked to the user. Should be a yes/no question.
    default : "yes"/"no"
        The presumed answer if the user just hits <Enter>.

    Returns
    -------
    bool
    """
    valid = {"yes": True, "y": True, "ye": True,
             "no": False, "n": False}
    if default is None:
        prompt = " [y/n] "
    elif default == "yes":
        prompt = " [Y/n] "
    elif default == "no":
        prompt = " [y/N] "
    else:
        raise ValueError("invalid default answer: '%s'" % default)

    while True:
        sys.stdout.write(question + prompt)
        choice = input().lower() # in Python 2.7, it is raw_input
        if default is not None and choice == '':
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            sys.stdout.write("Please respond with 'yes' or 'no' "
                             "(or 'y' or 'n').\n")
                             
def chunklist(l, n):
    """ Yield successive n-sized chunks from l."""
    for i in range(0, len(l), n):
        yield l[i:i+n]


def silent(function):
    """
    Decorate that silent the function it decorates,
    i.e SQL queries will not be printed.

    Notes
    -----
    * Legacy from Benchmark submodule

    """
    @wraps(function)
    def wrapper(self, *args, **kwds):
        original_value = os.environ['VERBOSE']
        set_verbose(False)
        result = function(self, *args, **kwds)
        os.environ['VERBOSE'] = original_value
        return result
    return wrapper

def to_nK(dataframe, nKrow):
    """
    Create a version of the dataframe that has n Krows.

    Parameters
    ----------
    dataframe : DataFrame
        DataFrame to use as a basis
    nKrow : int
        Number of Krows the return dataframe should contain

    Returns
    -------
    DataFrame
        A Version of the inputed dataframe with n Krows

    Notes
    -----

    * If the dataset is smaller than nKrow, it will be imputed randomly 
      with some existing rows, otherwise a random sample is returned
    * Legacy from Benchmark submodule

    """
    if nKrow < 1:
        raise ValueError("n should be an integer with minimum value 1")
    if not isinstance(nKrow, int):
        raise ValueError("n should be an integer with minimum value 1")
    def some(df, nrow):
        """Return n random rows from the given dataframe"""
        from numpy import random
        return df.ix[random.choice(range(len(df)), nrow)]

    if nKrow*1000 > dataframe.shape[0]:
        df = pd.concat([dataframe, some(dataframe, nKrow*1000 - dataframe.shape[0])], ignore_index=True)
    else:
        df = pd.concat([some(dataframe, nKrow*1000)], ignore_index=True)
    return df

def extend_dataset(df, n):
    """
    Extend a dataframe horizontaly by duplicating its columns n times

    Notes
    -----

    * Legacy from Benchmark submodule
    """
    df_2 = deepcopy(df)
    df_out = deepcopy(df)
    while n > 0:
        df_2.columns = ["%s_extended_%s"%(column,n) for column in df.columns]
        df_out = pd.concat([df_out,df_2], axis = 1)
        n -= 1
    return df_out

def check_tablename(tablename):
    """
    Check if a string is upper case. This function converts the string to upper 
    case and checks if it is a valid table name.

    Parameters
    ----------
    tablename : str
        string to be checked. 

    Returns
    -------
    str
        Checked and upper cased table name.

    Notes
    -----
        Table names should consist of upper case characters or numbers that can
        be separated by underscores (“_”) characters.
    """
    return _check_database_objectname(tablename, 'Table')

def check_viewname(viewname):
    """
    Convenience function for checking view names, which have the same prerequisites as table names.
    See the check_tablename documentation.
    """
    return _check_database_objectname(viewname, 'View')

def check_modelname(modelname):
    """
    Convenience function for checking model names, which have the same prerequisites as table names.
    See the check_tablename documentation.
    """
    return _check_database_objectname(modelname, 'Model')


def _check_case(name):
    """
    Check if the name given as parameter is in upper case and convert it to 
    upper cases.
    """
    if name != name.upper():
        warnings.warn("Mixed case names are not supported in database object names.", UserWarning)
    return name.upper()


def _check_database_objectname(tablename, objecttype):
    """
    Check if a string is upper case. This function converts the string to upper
    case and checks if it is a valid database object (table, view or model) name.

    Parameters
    ----------
    tablename : str
        string to be checked.

    objecttype : str
            ''

    Returns
    -------
    str
        Checked and upper cased database object name.

    Notes
    -----
        Database object names should consist of upper case characters or numbers that can
        be separated by underscores (“_”) characters.
    """
    tablename = _check_case(tablename)
    if not all([(char.isalnum() | (char == '_') | (char == '.')) for char in tablename]):
        raise ValueError("%s name '%s' is not valid, only alphanumeric characters and underscores are allowed."%(objecttype, tablename))
    if tablename.count(".") > 1:
        raise ValueError("%s name '%s' is not valid, only one '.' character is allowed."%(objecttype, tablename))
    return tablename


def _convert_dtypes(idadf, data):
    """
    DEPRECATED - CURRENTLY NOT IN USE
    was used for formatting dataframe types

    Convert datatypes in a dataframe to float or to int according to the 
    corresponding type in database. This function only works if the dataframe 
    that is given as a parameter has the same columns as the current 
    IdaDataFrame.
    """
    # Note : Here I made the choice to convert every numeric attributes to
    # float, the reason is that converting to int for an attribute that has
    # missing values leads to an error, this is known as a pandas' gotcha
    # We could use statistics._get_number_of_nas to know if there are
    # missing values for each attributes, but I made the choice no to do it,
    # this for reducing complexity and improve performance. However, it no
    # efficient in terms of memory.
    import pdb ; pdb.set_trace() ;
    original_dtype = idadf.dtypes

    int_attributes = ['SMALLINT', 'INTEGER', 'BIGINT', 'BOOLEAN']
    float_attributes = ['REAL', 'DOUBLE', 'FLOAT', 'DECIMAL',
                        'DECFLOAT', 'APPROXIMATE', 'NUMERIC']
    for index, dtype in enumerate(original_dtype.values):
        if dtype[0] in int_attributes + float_attributes:
            data[[data.columns[index]]] = data[[data.columns[index]]].astype(float)
        else:
            data[[data.columns[index]]] = data[[data.columns[index]]].astype(object)
    return data

def _reset_attributes(idaobject, attributes):
    """
    Delete an attribute of a list of attributes of an object, if the attribute exists.
    """
    if (not hasattr(attributes, "__iter__"))|isinstance(attributes, six.string_types):
        attributes = [attributes]
    for attribute in attributes:
        try:
            delattr(idaobject, attribute)
        except AttributeError:
            pass
        
def _check_input(idadf, target, features):
    """
    Check if the input is valid. 
    Target should be a string that exists as a column in the IdaDataFrame, or None.
    Features should a string or a list of columns in the IdaDataFrame, or None.
    If features is None, a list with all features different from target is returned
    """
    if target is not None:
        if not isinstance(target, six.string_types):
            raise ValueError("target should be a string")
        if target not in idadf.columns:
            raise ValueError("Unknown column %s"%target)
    
    if features is not None:
        if isinstance(features, six.string_types):
            if features not in idadf.columns:
                raise ValueError("Unknown column %s"%features)
            features = [features]
        for x in features:
            if x not in idadf.columns:
                raise ValueError("Unknown column %s"%x)
    else:
        if target is not None:
            features = [x for x in idadf.columns if x not in target]
        else:
            features = list(idadf.columns)
            
    return target, features